@@ -1,4 +1,10 @@
+#ifdef HAVE_CONFIG_H
 #include "config.h"
+#endif
+//apparantly some systems, such as cygwin, do not have getopt defined in unistd
+#ifdef HAVE_GETOPT_H
+#include <getopt.h>
+#endif
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
@@ -44,19 +50,19 @@ struct s_v11sub {
 	unsigned char tracknum;
 };
 struct s_id3{
-   char tag[3];
-   char songname[30];
-   char artist[30];
-   char album[30];
-   char year[4];
-   union{
-	   char comment[30];
-	   s_v11sub v11;
-   };
-//   char comment[28];
-//   unsigned char empty;
-//   unsigned char tracknum;
-   unsigned char genre;
+	char tag[3];
+	char songname[30];
+	char artist[30];
+	char album[30];
+	char year[4];
+	union{
+		char comment[30];
+		s_v11sub v11;
+	};
+	//char comment[28];
+	//unsigned char empty;
+	//unsigned char tracknum;
+	unsigned char genre;
 }id3;
 
 #define NUM_GENRE 148
@@ -95,7 +101,7 @@ const char *genre_list[NUM_GENRE+1]={
 	"Punk Rock", "Drum Solo", "A capella", "Euro-House", "Dance Hall",
 	"Goa", "Drum & Bass", "Club House", "Hardcore", "Terror",
 	"Indie", "BritPop", "NegerPunk", "Polsk Punk", "Beat",
-	"Christian Gangsta", "Heavy Metal", "Black Metal", "Crossover", "Contemporary C",
+	"Christian Gangsta", "Heavy Metal", "Black Metal", "Crossover", "Contemporary Christian",
 	"Christian Rock", "Merengue", "Salsa", "Thrash Metal", "Anime", "JPop",
 	"SynthPop",
 	NULL
@@ -105,21 +111,21 @@ void i3info(const char *file,int quiet);
 void i3remove(const char *file,int quiet);
 void i3rename(const char *file,int quiet,int test);
 void i3edit(const char * file, const char * defsongname,
-            const char * defartist, const char * defalbum,
-            const char * defyear, const char * defcomment,
-            const char * deftracknum, const char * defgenre,int quiet);
+		const char * defartist, const char * defalbum,
+		const char * defyear, const char * defcomment,
+		const char * deftracknum, const char * defgenre,int quiet);
 void stredit(const char * name, int maxlen, char * buf);
 void print_genre_list(int mode);
 
 void version()
 {
-      printf("id3ed v1.10.2 - mpeg layer 3 file information editor\n");
+	printf("id3ed v1.10.3 - mpeg layer 3 file information editor\n");
 }
 
 void license()
 {
-      printf("Copyright 1998-2000 Matt Mueller <donut@azstarnet.com>\n");
-      printf("\n\
+	printf("Copyright 1998-2001 Matthew Mueller <donut@azstarnet.com>\n");
+	printf("\n\
 This program is free software; you can redistribute it and/or modify\n\
 it under the terms of the GNU General Public License as published by\n\
 the Free Software Foundation; either version 2 of the License, or\n\
@@ -134,8 +140,8 @@ Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.\n");
 }
 
 void print_help(void){
-      version();
-      printf("\
+	version();
+	printf("\
 Usage: id3ed [-s songname] [-n artist] [-a album] [-y year] [-c comment]\n\
 	     [-k tracknum] [-g genre] [-q] [-SNAYCKG] [-l/-L] [-r]\n\
 	     [-i] <mp3files> [-v]\n\n\
@@ -147,122 +153,122 @@ Usage: id3ed [-s songname] [-n artist] [-a album] [-y year] [-c comment]\n\
   -r			remove id3 tag from files\n\
   -i			show current id3 tag only, don't edit\n\
   -v			output program version/license\n");
- }
+}
 
 int main(int argc,char ** argv){
-   int c,quiet=0,q2=0,mode=0;
-   char *songname = NULL, *artist = NULL, *album = NULL, *year = NULL,
-        *comment = NULL, *tracknum = NULL, *genre = NULL;
+	int c,quiet=0,alreadyquiet=0,mode=0;
+	char *songname = NULL, *artist = NULL, *album = NULL, *year = NULL,
+	*comment = NULL, *tracknum = NULL, *genre = NULL;
 
-   if (argc<2){
-	   print_help();
-   }
-   else {
-      while (EOF != (c = getopt(argc, argv, "s:n:a:y:c:k:g:qSNAYCKGlLrimMv"))){
-         switch (c){
-            case 's':
-               songname = optarg;
-               break;
-            case 'n':
-               artist = optarg;
-               break;
-            case 'a':
-               album = optarg;
-               break;
-            case 'y':
-               year = optarg;
-               break;
-            case 'c':
-               comment = optarg;
-               break;
-            case 'g':
-               genre = optarg;
-               break;
-            case 'k':
-               tracknum = optarg;
-               break;
-	    case 'q':
-#define REALQUIET() {if (!q2) { quiet = 0xffff & ~QUIET_PROGRESS; q2=1;}}
+	if (argc<2){
+		print_help();
+	}
+	else {
+		while (EOF != (c = getopt(argc, argv, "s:n:a:y:c:k:g:qSNAYCKGlLrimMv"))){
+			switch (c){
+				case 's':
+					songname = optarg;
+					break;
+				case 'n':
+					artist = optarg;
+					break;
+				case 'a':
+					album = optarg;
+					break;
+				case 'y':
+					year = optarg;
+					break;
+				case 'c':
+					comment = optarg;
+					break;
+				case 'g':
+					genre = optarg;
+					break;
+				case 'k':
+					tracknum = optarg;
+					break;
+				case 'q':
+#define REALQUIET() {if (!alreadyquiet) { quiet = 0xffff & ~QUIET_PROGRESS; alreadyquiet=1;}}
 #define UNQUIET(b) { REALQUIET(); quiet &= ~b;}
-	       if(q2)
-		       quiet |= QUIET_PROGRESS;
-	       else {
-			   REALQUIET();
-	       }
-	       break;
-	    case 'S':
-	       UNQUIET(QUIET_SONG);
-	       break;
-	    case 'N':
-	       UNQUIET(QUIET_ARTIST);
-	       break;
-	    case 'A':
-	       UNQUIET(QUIET_ALBUM);
-	       break;
-	    case 'Y':
-	       UNQUIET(QUIET_YEAR);
-	       break;
-	    case 'C':
-	       UNQUIET(QUIET_COMMENT);
-	       break;
-	    case 'G':
-	       UNQUIET(QUIET_GENRE);
-	       break;
-	    case 'K':
-	       UNQUIET(QUIET_TRACK);
-	       break;
-	    case 'l':
-	       print_genre_list(0);
-	       break;
-	    case 'L':
-	       print_genre_list(1);
-	       break;
-	    case 'r':
-	       mode=1;
-	       break;
-	    case 'i':
-	       mode=2;
-	       break;
-	    case 'v':
-	       version();
-		   license();
-	       break;
-            case ':':
-            case '?':
-//               fprintf(stderr, "Illegal parameters\n");
-	       print_help();
-               return 1;
-		  case 'm':
-			mode=3;
-			break;
-		  case 'M':
-			mode=4;
-			break;
-         }
-      }
-//      printf("quiet=%i q2=%i\n",quiet,q2);
-      for (int i=optind;i<argc;i++)
-	      switch (mode){
-			  case 3:
-			  case 4:
-				  i3rename(argv[i],quiet,mode==3);
-				  break;
-			  case 2:
-				  i3info(argv[i],quiet);
-				  break;
-			  case 1:
-				  i3remove(argv[i],quiet);
-				  break;
-			  default:
-				  i3edit(argv[i],songname,artist,album,year,comment,tracknum,genre,quiet);
-		  }
-   }
-   return 0;
+					if(alreadyquiet)
+						quiet |= QUIET_PROGRESS;
+					else {
+						REALQUIET();
+					}
+					break;
+				case 'S':
+					UNQUIET(QUIET_SONG);
+					break;
+				case 'N':
+					UNQUIET(QUIET_ARTIST);
+					break;
+				case 'A':
+					UNQUIET(QUIET_ALBUM);
+					break;
+				case 'Y':
+					UNQUIET(QUIET_YEAR);
+					break;
+				case 'C':
+					UNQUIET(QUIET_COMMENT);
+					break;
+				case 'G':
+					UNQUIET(QUIET_GENRE);
+					break;
+				case 'K':
+					UNQUIET(QUIET_TRACK);
+					break;
+				case 'l':
+					print_genre_list(0);
+					break;
+				case 'L':
+					print_genre_list(1);
+					break;
+				case 'r':
+					mode=1;
+					break;
+				case 'i':
+					mode=2;
+					break;
+				case 'v':
+					version();
+					license();
+					break;
+				case ':':
+				case '?':
+					//fprintf(stderr, "Illegal parameters\n");
+					print_help();
+					return 1;
+				case 'm':
+					mode=3;
+					break;
+				case 'M':
+					mode=4;
+					break;
+			}
+		}
+		//ntf("quiet=%i alreadyquiet=%i\n",quiet,alreadyquiet);
+		for (int i=optind;i<argc;i++)
+			switch (mode){
+				case 3:
+				case 4:
+					i3rename(argv[i],quiet,mode==3);
+					break;
+				case 2:
+					i3info(argv[i],quiet);
+					break;
+				case 1:
+					i3remove(argv[i],quiet);
+					break;
+				default:
+					i3edit(argv[i],songname,artist,album,year,comment,tracknum,genre,quiet);
+			}
+	}
+	return 0;
 }
 
 void print_genre_list(int mode){
 	int c=0,w=0;
-#ifdef HAVE_LIBREADLINE
+#ifdef HAVE_LIBCURSES
 	c=COLS;
 #endif
 	if (c<=0)c=80;
@@ -311,9 +317,9 @@ int agenretoi(const char * g){
 char *my_rl_default;
 
 int set_rl_default(void){
-   rl_insert_text(my_rl_default);
-//   rl_forced_update_display();
-   return 0;
+	rl_insert_text(my_rl_default);
+	//rl_forced_update_display();
+	return 0;
 }
 #ifdef HAVE_SET_H
 #include <set.h>
@@ -426,12 +432,19 @@ void stredit(const char * name, int maxlen, char * buf){
 //##### TODO: make renaming configurable?
 void i3rename(const char *file,int quiet,int test){
 	int f;
-	off_t end;
 	if (doopen(f,file,RDMODE))return;
 	if ((!quiet) || test){
 		printf("%s->",file);fflush(stdout);
 	}
-	if ((end=lseek(f,-128,SEEK_END))<0){
+	off_t end=lseek(f,0,SEEK_END);
+	if (end>=0 && end<128) {
+		close (f);
+		if ((!quiet))
+			printf("(no tag)\n");
+		return;
+	}
+	if (lseek(f,-128,SEEK_END)<0){
+		close (f);
 		perror("lseek");return;
 	}
 	if (doread(f,&id3,sizeof(id3),"id3buf"))return;
@@ -456,10 +469,17 @@ void i3rename(const char *file,int quiet,int test){
 
 void i3info(const char *file,int quiet){
 	int f;
-	off_t end;
 	if (doopen(f,file,RDMODE))return;
 	printf("%s: ",file);fflush(stdout);
-	if ((end=lseek(f,-128,SEEK_END))<0){
+	off_t end=lseek(f,0,SEEK_END);
+	if (end>=0 && end<128) {
+		close (f);
+		if ((!quiet))
+			printf("(no tag)\n");
+		return;
+	}
+	if (lseek(f,-128,SEEK_END)<0){
+		close (f);
 		perror("lseek");return;
 	}
 	if (doread(f,&id3,sizeof(id3),"id3buf"))return;
@@ -487,9 +507,16 @@ void i3info(const char *file,int quiet){
 
 void i3remove(const char *file,int quiet){
 	int f;
-	off_t end;
 	if (doopen(f,file,RDWRMODE))return;
+	off_t end=lseek(f,0,SEEK_END);
+	if (end>=0 && end<128) {
+		close (f);
+		if ((!quiet))
+			printf("no tag in %s\n",file);
+		return;
+	}
 	if ((end=lseek(f,-128,SEEK_END))<0){
+		close (f);
 		perror("lseek");return;
 	}
 	if (doread(f,&id3,sizeof(id3),"id3buf"))return;
@@ -505,109 +532,120 @@ void i3remove(const char *file,int quiet){
 }
 
 void i3edit(const char * file, const char * defsongname,
-            const char * defartist, const char * defalbum,
-            const char * defyear, const char * defcomment,
-            const char * deftracknum, const char * defgenre, int quiet){
-   int f,hasid3,commentsize=28;
-   if (!(quiet&QUIET_PROGRESS))
-	   printf("\nFile %s: ", file);
-   if (doopen(f,file,RDWRMODE))return;
-   if (lseek(f,-128,SEEK_END)<0){
-      perror("lseek");return;
-   }
-   if (doread(f,&id3,sizeof(id3),"id3buf"))return;
-   if (strncmp(id3.tag,"TAG",3)){
-      memset(&id3,0,sizeof(id3));
-      id3.tag[0]='T';id3.tag[1]='A';id3.tag[2]='G';
-      id3.genre=255;
-      hasid3=0;
-	  if (!(quiet&QUIET_PROGRESS))
-		  printf("(no tag)\n");
-   } else {
-		if (id3.v11.empty==0){
+		const char * defartist, const char * defalbum,
+		const char * defyear, const char * defcomment,
+		const char * deftracknum, const char * defgenre, int quiet){
+	int f,hasid3,commentsize=28;
+	if (!(quiet&QUIET_PROGRESS))
+		printf("\nFile %s: ", file);
+	if (doopen(f,file,RDWRMODE))return;
+	off_t end=lseek(f,0,SEEK_END);
+	if (end>=0 && end<128) {
+		memset(&id3,0,sizeof(id3));
+		id3.tag[0]='T';id3.tag[1]='A';id3.tag[2]='G';
+		id3.genre=255;
+		hasid3=0;
+		if (!(quiet&QUIET_PROGRESS))
+			printf("(no tag)\n");
+	} else {
+		if (lseek(f,-128,SEEK_END)<0){
+			close(f);
+			perror("lseek");return;
+		}
+		if (doread(f,&id3,sizeof(id3),"id3buf"))return;
+		if (strncmp(id3.tag,"TAG",3)){
+			memset(&id3,0,sizeof(id3));
+			id3.tag[0]='T';id3.tag[1]='A';id3.tag[2]='G';
+			id3.genre=255;
+			hasid3=0;
 			if (!(quiet&QUIET_PROGRESS))
-				printf("(tag v1.1)\n");
-			hasid3=2;//id3 v1.1
-		}else{
-			if (!(quiet&1))
-				printf("(tag v1.0");
-			if (deftracknum){
-				hasid3=2;
+				printf("(no tag)\n");
+		} else {
+			if (id3.v11.empty==0){
 				if (!(quiet&QUIET_PROGRESS))
-					printf("->1.1)\n");
+					printf("(tag v1.1)\n");
+				hasid3=2;//id3 v1.1
 			}else{
-				hasid3=1;//id3 v1.0
-				commentsize=30;
-				if (!(quiet&QUIET_PROGRESS))
-					printf(")\n");
+				if (!(quiet&1))
+					printf("(tag v1.0");
+				if (deftracknum){
+					hasid3=2;
+					if (!(quiet&QUIET_PROGRESS))
+						printf("->1.1)\n");
+				}else{
+					hasid3=1;//id3 v1.0
+					commentsize=30;
+					if (!(quiet&QUIET_PROGRESS))
+						printf(")\n");
+				}
 			}
 		}
-   }
-    if (defsongname){
-       memset(id3.songname,0,sizeof(id3.songname));
-       strncpy(id3.songname,defsongname,sizeof(id3.songname));
-    }
-    if (defartist){
-       memset(id3.artist,0,sizeof(id3.artist));
-       strncpy(id3.artist,defartist,sizeof(id3.artist));
-    }
-    if (defalbum){
-       memset(id3.album,0,sizeof(id3.album));
-       strncpy(id3.album,defalbum,sizeof(id3.album));
-    }
-    if (defyear){
-       memset(id3.year,0,sizeof(id3.year));
-       strncpy(id3.year,defyear,sizeof(id3.year));
-    }
-    if (defcomment){
-       memset(id3.comment,0,commentsize);
-       strncpy(id3.comment,defcomment,commentsize);
-    }
-    if (deftracknum){
-       //memset(id3.tracknum,0,sizeof(id3.tracknum));
-       id3.v11.empty=0;
-       id3.v11.tracknum = atoi(deftracknum);
-       //strncpy(id3.tracknum,deftracknum,sizeof(id3.tracknum));
-    }
-    if (defgenre){
-	 id3.genre = agenretoi(defgenre);
-//       unsigned tmp;
-//       sscanf(defgenre,"%u",&tmp);
-//       id3.genre = tmp;
-    }
-   if (!(quiet&QUIET_SONG))
-	   stredit("songname",30,id3.songname);
-   if (!(quiet&QUIET_ARTIST))
-	   stredit("artist",30,id3.artist);
-   if (!(quiet&QUIET_ALBUM))
-	   stredit("album",30,id3.album);
-   if (!(quiet&QUIET_YEAR))
-	   stredit("year",4,id3.year);
-   if (!(quiet&QUIET_COMMENT))
-	   stredit("comment",commentsize,id3.comment);
-   if (hasid3!=1 && !(quiet&QUIET_TRACK)) {
-	   char tbuf[3];
-	   sprintf(tbuf, "%i", id3.v11.tracknum);
-	   stredit("tracknum", 3, tbuf);
-	   id3.v11.empty=0;
-	   id3.v11.tracknum=atoi(tbuf);
-   }
-   if (!(quiet&QUIET_GENRE)){
-//	   char sbuf[4];
-//	   sprintf(sbuf,"%i",id3.genre);
-//	   stredit("genre",3,sbuf);
-//	   id3.genre=atoi(sbuf);
-	   genreedit(id3.genre);
-   }
-   if (hasid3){
-      if (lseek(f,-128,SEEK_END)<0){
-	 perror("lseek");return;
-      }
-   }else{
-      if (lseek(f,0,SEEK_END)<0){
-	 perror("lseek");return;
-      }
-   }
-   if (dowrite(f,&id3,sizeof(id3),"id3buf"))return;
-   close(f);
+	}
+	if (defsongname){
+		memset(id3.songname,0,sizeof(id3.songname));
+		strncpy(id3.songname,defsongname,sizeof(id3.songname));
+	}
+	if (defartist){
+		memset(id3.artist,0,sizeof(id3.artist));
+		strncpy(id3.artist,defartist,sizeof(id3.artist));
+	}
+	if (defalbum){
+		memset(id3.album,0,sizeof(id3.album));
+		strncpy(id3.album,defalbum,sizeof(id3.album));
+	}
+	if (defyear){
+		memset(id3.year,0,sizeof(id3.year));
+		strncpy(id3.year,defyear,sizeof(id3.year));
+	}
+	if (defcomment){
+		memset(id3.comment,0,commentsize);
+		strncpy(id3.comment,defcomment,commentsize);
+	}
+	if (deftracknum){
+		//memset(id3.tracknum,0,sizeof(id3.tracknum));
+		id3.v11.empty=0;
+		id3.v11.tracknum = atoi(deftracknum);
+		//strncpy(id3.tracknum,deftracknum,sizeof(id3.tracknum));
+	}
+	if (defgenre){
+		id3.genre = agenretoi(defgenre);
+		//unsigned tmp;
+		//sscanf(defgenre,"%u",&tmp);
+		//id3.genre = tmp;
+	}
+	if (!(quiet&QUIET_SONG))
+		stredit("songname",30,id3.songname);
+	if (!(quiet&QUIET_ARTIST))
+		stredit("artist",30,id3.artist);
+	if (!(quiet&QUIET_ALBUM))
+		stredit("album",30,id3.album);
+	if (!(quiet&QUIET_YEAR))
+		stredit("year",4,id3.year);
+	if (!(quiet&QUIET_COMMENT))
+		stredit("comment",commentsize,id3.comment);
+	if (hasid3!=1 && !(quiet&QUIET_TRACK)) {
+		char tbuf[3];
+		sprintf(tbuf, "%i", id3.v11.tracknum);
+		stredit("tracknum", 3, tbuf);
+		id3.v11.empty=0;
+		id3.v11.tracknum=atoi(tbuf);
+	}
+	if (!(quiet&QUIET_GENRE)){
+		//char sbuf[4];
+		//sprintf(sbuf,"%i",id3.genre);
+		//stredit("genre",3,sbuf);
+		//id3.genre=atoi(sbuf);
+		genreedit(id3.genre);
+	}
+	if (hasid3){
+		if (lseek(f,-128,SEEK_END)<0){
+			perror("lseek");return;
+		}
+	}else{
+		if (lseek(f,0,SEEK_END)<0){
+			perror("lseek");return;
+		}
+	}
+	if (dowrite(f,&id3,sizeof(id3),"id3buf"))return;
+	close(f);
 }
